import datetime
import json
import qrcode
from io import BytesIO
import base64


class qr_payments:

    code = None
    qr_code = None
    value = None
    phone_number = None
    type = None
    message_id = None
    reference_1 = None
    reference_2 = None

    def get_destination(self, service):
        destination = {}
        destination['ServiceName'] = service.name
        destination['ServiceOperation'] = service.operation
        destination['ServiceRegion'] = service.region
        destination['ServiceVersion'] = service.version
        return destination

    def get_header(self, service, destination):
        headers = {}
        headers['Channel'] = service.channel
        headers['RequestDate'] = service.request_date
        headers['MessageID'] = service.message_id
        headers['ClientID'] = service.client_id

        headers['Destination'] = destination

        return headers

    def get_code_qr(self):
        params = {}
        params['code'] = self.code
        params['value'] = self.value
        params['reference1'] = self.reference_1
        if(self.reference_2):
            params['reference2'] = self.reference_2
        
        generate_code_qrrq = {'generateCodeQRRQ':params}
        body = {}
        body['any'] = generate_code_qrrq
        return body
    
    def get_status(self):
        params = {}
        params['codeQR'] = self.qr_code
        get_status_payment_rq = {'getStatusPaymentRQ':params}
        body = {}
        body['any'] = get_status_payment_rq
        return body
    
    def reverse(self):
        params = {}
        params['code'] = self.qr_code
        params['value'] = self.qr_code
        params['phoneNumber'] = self.phone_number
        params['type'] = self.type
        params['MessageID'] = self.message_id
        get_status_payment_rq = {'reversionRQ':params}
        body = {}
        body['any'] = get_status_payment_rq
        return body

    def get_request_message(self, headers, request):
        request_message = {
                                "RequestMessage": {
                                                    "RequestHeader": headers,
                                                    "RequestBody": request
                                                  }
                          }
        
        return request_message
    
    def get_qr_image_source(self, code):
        qr = qrcode.QRCode(
                            version=1,
                            error_correction=qrcode.constants.ERROR_CORRECT_L,
                            box_size=20,
                            border=4,
                          )
        qr.add_data(code)
        qr.make(fit=True)
        img = qr.make_image()
        stream_out = BytesIO()
        img.save(stream_out, format="PNG")
        source = base64.b64encode(stream_out.getvalue())
        return str('data:image/png;base64,') + str(source.decode("utf-8")) 