from . utils.endpoints import endpoints
from . utils.credentials import credentials
from . qr.qr_payments import qr_payments


import datetime
import logging
import sys
import json
import requests
import base64
_logger = logging.getLogger(__name__)


class service:
    qr_payment_manager = None

    # destination
    mode = None
    name = None
    operation = None
    region = None
    version = None
    # headers
    channel = None
    client_id = None
    request_date = None
    message_id = None

    _credentials = None

    def __init__(self, params):
        try:
            self.mode = params['mode']
            self.name = params['name']
            self.region = params['region']
            self.version = params['version']
            self.operation = params['operation']

            self.channel = params['channel']
            self.client_id = params['client_id']

            self._credentials = credentials()
            self._credentials.mode = params['mode']
            self._credentials.set_api_key(params['api_key'])
            self._credentials.set_access_key(params['access_key'])
            self._credentials.set_client_id(params['client_id'])
        except Exception as e:
            _logger.error('\n')
            _logger.error(
                'BG __init__ service *********************************')
            _logger.error(getattr(e, 'message', repr(e)))
            _logger.error(str('line: ') +
                          str((format(sys.exc_info()[-1].tb_lineno))))
            _logger.error(
                'EOF __init__ service ********************************')
            _logger.error('\n')

    def init_qr_payments(self):
        self.qr_payment_manager = qr_payments()

    def get_body(self):
        body = None
        if(str(self.name) == str('PaymentsService')):
            # BG QR payments
            if(str(self.operation) == str('generateCodeQR')
                    or str(self.operation) == str('getStatusPayment')
                    or str(self.operation) == str('reverseTransaction')):
                # EOF QR payments
                destination = self.qr_payment_manager.get_destination(self)
                self.get_new_request_date()
                headers = self.qr_payment_manager.get_header(self, destination)
                request = None
                if(str(self.operation) == str('generateCodeQR')):
                    request = self.qr_payment_manager.get_code_qr()
                if(str(self.operation) == str('getStatusPayment')):
                    request = self.qr_payment_manager.get_status()
                if(str(self.operation) == str('reverseTransaction')):
                    request = self.qr_payment_manager.reverse()
                body = self.qr_payment_manager.get_request_message(
                    headers, request)

        return body

    def make_request(self, body):
        _endpoints = endpoints()
        _url = str()
        if(str(self.operation) == str('generateCodeQR')):
            if(str(self.mode) == str('sandbox')):
                _url = str(_endpoints._urls['sandbox']) + \
                    str(_endpoints._urls['qr_image'])
            else:
                _url = str(_endpoints._urls['production']) + \
                    str(_endpoints._urls['qr_image'])

        if(str(self.operation) == str('getStatusPayment')):
            if(str(self.mode) == str('sandbox')):
                _url = str(_endpoints._urls['sandbox']) + \
                    str(_endpoints._urls['qr_payment_status'])
            else:
                _url = str(_endpoints._urls['production']) + \
                    str(_endpoints._urls['qr_payment_status'])
        authorization = 'Bearer {}'.format(self._credentials.token)
        _headers = {}
        _headers['Authorization'] = authorization
        _headers['Accept'] = 'application/json'
        _headers['Content-Type'] = 'application/json'
        _headers['x-api-key'] = self._credentials.credentials['api_key']
        _logger.warning('_headers')
        _logger.warning(_headers)
        response = requests.post(
            url=_url, headers=_headers, data=json.dumps(body))
        response = json.loads(response.text)
        _logger.warning(
            'BG make_request response *********************************')
        _logger.warning(response)
        _logger.warning(
            'EOF make_request response ********************************')
        return response

    def handle_response(self, response):
        short_response = {'short_status': 'fail','sub_status': 'fail'}
        qr_code = None
        error_message = None
        if('ResponseMessage' in response):
            if('ResponseHeader' in response['ResponseMessage']):
                if('Status' in response['ResponseMessage']['ResponseHeader']):
                    status = response['ResponseMessage']['ResponseHeader']['Status']
                    if('StatusDesc' in status):
                        if(str(status['StatusDesc']) == str('SUCCESS')):
                            _logger.warning('001')
                            short_response['sub_status'] = 'ok'
                            if('ResponseBody' in response['ResponseMessage']):
                                if('generateCodeQRRS' in response['ResponseMessage']['ResponseBody']['any']):
                                    _logger.warning('002')
                                    if('generateCodeQRRS' in response['ResponseMessage']['ResponseBody']['any']):
                                        if('codeQR' in response['ResponseMessage']['ResponseBody']['any']['generateCodeQRRS']):
                                            _logger.warning('003')
                                            qr_code = response['ResponseMessage']['ResponseBody']['any']['generateCodeQRRS']['codeQR']
                                            short_response['short_status'] = 'ok'
                                            short_response['sub_status'] = 'ok'
                                            short_response['qr_code'] = qr_code

                                if('getStatusPaymentRS' in response['ResponseMessage']['ResponseBody']['any']):
                                    short_response['short_status'] = 'ok'
                                    short_response['sub_status'] = 'ok'
                                    payment_response = {}
                                    if('status' in response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']):
                                        payment_response['status'] = response['ResponseMessage'][
                                            'ResponseBody']['any']['getStatusPaymentRS']['status']
                                    if('name' in response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']):
                                        payment_response['name'] = response['ResponseMessage'][
                                            'ResponseBody']['any']['getStatusPaymentRS']['name']
                                    if('value' in response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']):
                                        payment_response['value'] = response['ResponseMessage'][
                                            'ResponseBody']['any']['getStatusPaymentRS']['value']
                                    if('date' in response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']):
                                        payment_response['date'] = response['ResponseMessage'][
                                            'ResponseBody']['any']['getStatusPaymentRS']['date']
                                    if('trnId' in response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']):
                                        payment_response['transaction_id'] = response['ResponseMessage'][
                                            'ResponseBody']['any']['getStatusPaymentRS']['trnId']
                                    if('ipAddress' in response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']):
                                        payment_response['ip'] = response['ResponseMessage'][
                                            'ResponseBody']['any']['getStatusPaymentRS']['ipAddress']
                                    if('originMoney' in response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']):
                                            payment_response['origin_money'] = None
                                            origin_money = response['ResponseMessage']['ResponseBody']['any']['getStatusPaymentRS']['originMoney']
                                            payment_response['origin_money'] = origin_money
                                    short_response['payment_response'] = payment_response

                                if('reversionRQ' in response['ResponseMessage']['ResponseBody']['any']):                                    
                                    if('getStatusPaymentRQ' in response['ResponseMessage']['ResponseBody']['any']):
                                        if('getStatusPaymentRQ' in response['ResponseMessage']['ResponseBody']['any']):
                                            reversion_rq = response['ResponseMessage']['ResponseBody']['any']['reversionRQ']
                                            short_response['reversion'] = reversion_rq  
                                            short_response['short_status'] = 'ok'
                                            short_response['sub_status'] = 'ok' 
                        
                        else:
                            short_response['error_message'] = str(status['StatusDesc'])
                            short_response['sub_status'] = 'fail'
        return short_response

    def get_new_request_date(self):
        _datetime_now = datetime.datetime.utcnow()
        self.message_id = _datetime_now.strftime('%Y%m%d')
        self.request_date = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.000Z")
